#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <limits.h>
#include <errno.h>
#include "flowdata.h"
#include "fdtools.h"
#include <time.h>
#include <unistd.h>
#include <libpq-fe.h>

char * flow_ptr = 0;
char bin_name[PATH_MAX];
FILE *logf;
char	*pghost,
   	*pgport,
    	*pgoptions,
    	*pgtty;
char	*dbName;
PGconn	*conn;
PGresult *res;
void exit_nicely(PGconn *conn);
void cleanUp (int i);
void sigHandler (int sig);
void GetFlow (int soc, struct sockaddr_in *socadr);

void exit_nicely(PGconn *conn)
{
    PQfinish(conn);
    exit(1);
}
 
void cleanUp (i)
int i;
{
    PQfinish(conn);
    if (i) {
        fprintf(logf, "exiting\n");
    }
    free(flow_ptr);
    exit(i);
}

void sigHandler (sig)
int sig;
{
    fprintf(logf, "Received signal %i\n", sig);
    cleanUp(sig);
}

void GetFlow (soc, socadr)
int soc;
struct sockaddr_in * socadr;
{
    char buf[1024];
    int cnt, alen, flowcount = 0, flowrec = 0;
    ushort header_count = 0;
    struct in_addr saddr, daddr, scaddr;
    struct tm *time;
    long tmplong;
    char strptr1[16], strptr2[16], strptr3[16];
    IPStat5Msg * v5_flow = (IPStat5Msg *) flow_ptr;
    alen = sizeof(struct sockaddr_in);

 conn = PQsetdb(pghost, pgport, pgoptions, pgtty, dbName);
 if (PQstatus(conn) == CONNECTION_BAD)
     {
     	fprintf(logf, "Connection to database '%s' failed.\n", dbName);
        fprintf(logf, "%s", PQerrorMessage(conn));
        exit_nicely(conn);
     }
while (1) {
        if (recvfrom(soc, flow_ptr, MAX_FLOW_PAK_SIZE, 0, 
                     (struct sockaddr *)socadr, &alen) < 0){
    fprintf(logf, "%s: recvfrom() failed, %s\n", bin_name, strerror(errno));
    cleanUp(1);}
        if((ntohs(*(ushort *)flow_ptr))==FLOW_VERSION_5) {
              header_count = ntohs(v5_flow->header.count);
              if (header_count > V5FLOWS_PER_PAK) {
                  fprintf(logf, "%s: Invalid flow count\n", bin_name);
                  cleanUp(1);
                                                  } else {
                      flowrec += header_count;
                      tmplong = ntohl(v5_flow->header.unix_secs);
		      time = localtime(&tmplong);
		      cnt = v5_flow->header.engine_type;
	    if((cnt != V5_RSP_EXPORT) || (cnt != V5_RSP_EXPORT))
                        {
                          fprintf(logf, "Received a version 5 flow from %s, "
                                          "with unknown engine_type %u\n",
                                          inet_ntoa(socadr->sin_addr),
                                          v5_flow->header.engine_type);
                          cleanUp(1);
                        }
	
// FOR COUNTER
 for (cnt = 0; cnt < header_count; cnt++) {
          saddr.s_addr = v5_flow->records[cnt].srcaddr;
          daddr.s_addr = v5_flow->records[cnt].dstaddr;
          strcpy(strptr1, inet_ntoa(saddr));
          strcpy(strptr2, inet_ntoa(daddr));

// INSERT COMMAND
insert:
    sprintf(buf,"INSERT INTO data_%d_%d (Day,Hour,Min,Sec,SrcIP,DstIP,SPort,DPort,\
		    InInt,OutInt,Prot,Pkts,Octets,SrcAS,DstAS) VALUES \
               	 ('%d','%d','%d','%d','%s','%s', %u, %u,'%u','%u','%u', %u, %u, %u, %u)",
              time->tm_year + 1900,
              time->tm_mon + 1,
              time->tm_mday,
              time->tm_hour,
              time->tm_min,
	      time->tm_sec,
              strptr1,
              strptr2,
              ntohs(v5_flow->records[cnt].srcport),
              ntohs(v5_flow->records[cnt].dstport),
              ntohs(v5_flow->records[cnt].input),
              ntohs(v5_flow->records[cnt].output),
        	 v5_flow->records[cnt].prot,
              ntohl(v5_flow->records[cnt].dPkts),
              ntohl(v5_flow->records[cnt].dOctets),
              ntohs(v5_flow->records[cnt].src_as),
              ntohs(v5_flow->records[cnt].dst_as));
	res = PQexec(conn, buf);
	
if(res == NULL || PQresultStatus(res) != PGRES_COMMAND_OK){
    if(!strstr((char*)res, "Table does not exist.")){
// CREATE TABLE COMMAND
	sprintf(buf, "CREATE TABLE data_%d_%d(  Id	SERIAL PRIMARY KEY,
						Day	VARCHAR(2),
    						Hour	VARCHAR(2),
    						Min	VARCHAR(2),
    						Sec	VARCHAR(2),
    						SrcIP	TEXT, 
    						DstIP	TEXT,
    						SPort	INT4, 
    						DPort	INT4,
    						InInt	VARCHAR(1), 
    						OutInt	VARCHAR(1), 
    						Prot	VARCHAR(1), 
    						Pkts	INT4, 
    						Octets	INT4,
						SrcAS	INT4,
						DstAS	INT4 )",
    						time->tm_year + 1900,
    						time->tm_mon + 1);
	res = PQexec(conn, buf);				
	if(res == NULL || PQresultStatus(res) != PGRES_COMMAND_OK)
	    {
		fprintf(logf, "CREATE  command failed :%s:\n",buf);
		if (res) PQclear(res);
		exit_nicely(conn);
	    }
	PQclear(res);
// CREATE TABLE COMMAND END
// GRAND COMMAND
	sprintf(buf,"GRANT SELECT ON data_%d_%d TO nobody",
				time->tm_year + 1900,
				time->tm_mon + 1);
	res = PQexec(conn, buf);				
    	if(res == NULL || PQresultStatus(res) != PGRES_COMMAND_OK)
	    {
		fprintf(logf,"GRANT  command failed\n");
		if (res) PQclear(res);
        	exit_nicely(conn);
	    }
	PQclear(res);
// GRAND COMMAND END
	goto insert;
	}
    else
	{
	    fprintf(logf, "INSERT  command failed\n");
	    if (res) PQclear(res);
	    exit_nicely(conn);
	}
    PQclear(res);
    }
// INSERT COMMAND END
}
// FOR COUNTER END
  }
	}
    else
            {
              fprintf(logf, "%s: Received datagram with unknown or invalid "
                                  "version number = %u\n", 
                      bin_name, ntohs(*((ushort *)flow_ptr)));
              cleanUp(1);
    	    }
    }
} 

int main(argc, argv)
int argc;
char *argv[];
{
    int soc;
    int socketBufSize = DEF_SOCKET_BUFFER_SIZE;
    ushort port = SERV_UDP_PORT;
    struct sockaddr_in socadr;
    
   logf=fopen("/var/log/trafd.log","a");
   if (fork()) return 0;
   if (setsid() < 0)
   {
   	  fprintf(stderr, "trafd: ");
   	  perror("cannot disassociate from controlling TTY");
   	  exit(1);
   }
    pghost = NULL;
    pgport = NULL;
    pgoptions = NULL;
    pgtty = NULL;
    dbName = "traffic";                                                                 

    signal(SIGINT, sigHandler);
    signal(SIGBUS, sigHandler);
    signal(SIGFPE, sigHandler);
    signal(SIGIOT, sigHandler);
    signal(SIGTERM, sigHandler);
    signal(SIGSEGV, sigHandler);

#ifdef __SVR4
    memset((char *)&socadr, 0, sizeof(struct sockaddr_in));
#else
    bzero((char *) &socadr, sizeof(struct sockaddr_in));
#endif
    socadr.sin_family = AF_INET;
    socadr.sin_addr.s_addr = htonl(INADDR_ANY);

    socadr.sin_port = htons(port);
    flow_ptr = (char *) malloc(MAX_FLOW_PAK_SIZE);
    if ((soc = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
        fprintf(logf, "%s: socket() failed, %s\n", bin_name, strerror(errno));
        cleanUp(1);
    }
    if (bind(soc, (struct sockaddr *)&socadr, sizeof(struct sockaddr_in)) < 0) {
        fprintf(logf, "%s: bind() failed, %s\n", bin_name, strerror(errno));
        cleanUp(1);
    }
    if (setsockopt(soc, SOL_SOCKET, SO_RCVBUF, (char *)&socketBufSize, 
                   sizeof(socketBufSize)) < 0) {
        fprintf(logf,"%s: setsockopt() failed, %s\n", bin_name, 
                                                        strerror(errno));
        cleanUp(1);
    }
    fprintf(logf,"\n Listening on %u\n", port);
    fflush(logf);
    GetFlow(soc, &socadr);
    return 0;
}
